/*
 * Copyright (c) 2015-2021 by inovat
 * innovative systeme - verkehr - tunnel - technik,
 * Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer
 *
 * de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer.
 *
 * de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 * de.bsvrz.kex.tls.osi7.DeBlocksMultiBlockTransfer wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 *
 * Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 * Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.sys.funclib.debug.Debug;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

//~ KLASSEN ===================================================================

/**
 * Erm&ouml;glicht den Zugriff auf FG/Typ/ID Kombinationen, f&uuml;r die die Daten mit mehreren DE-Bl&ouml;cken
 * und der gleichen Jobnummer versendet werden m&uuml;ssen; z. B. TLS2009 Dateitransfers (FG254, Typ 41).<br>
 * Die Definition einer FG/Typ/ID Kombination in dieser Klasse f&uuml;hrt dazu, dass die einzelnen DE-Bl&ouml;cke
 * zun&auml;chst gesammelt werden, bis sie vollständig sind. Dann werden alle diese Bl&ouml;cke mit der gleichen
 * Jobnummer versendet.
 * Die Struktur der DE-Bl&ouml;cke muss bez&uuml;glich der Blocknummern-Informationen wie folgt aufgebaut sein:<br>
 * (entsprechend TLS2009 Dateitransfers, FG254, Typ 41:<br>
 *      <pre>
 * Erster Block:
 *
 * Position         Bezeichnung                             Erl&auml;uterung
 * Byte 1           L&auml;nge DE-Block                     L&auml;nge des folgenden DE-Blocks
 * Byte 2           Daten-Endger&auml;te-Kanal              [ 0..255 ]
 * Byte 3           Typ der DE-Daten
 * Byte 4           Blocknummer low Byte                    [ 1 ]
 * Byte 5           Blocknummer high Byte
 * Byte 6           Anzahl Datenbl&ouml;cke low Byte        [ 2..65535 ]
 * Byte 7           Anzahl Datenbl&ouml;cke high Byte
 * Byte 8                                                   Daten
 *
 * Weitere Bl&ouml;cke:
 *
 * Position         Bezeichnung                             Erl&auml;uterung
 * Byte 1           L&auml;nge DE-Block                     L&auml;nge des folgenden DE-Blocks
 * Byte 2           Daten-Endger&auml;te-Kanal              [ 0..255 ]
 * Byte 3           Typ der DE-Daten
 * Byte 4           Blocknummer low Byte                    [ 0,2..65535 ]
 * Byte 5           Blocknummer high Byte
 * Byte 6                                                   Daten
 * </pre>
 *
 * Weiterhin gelten die Festlegungen TLS2009 Dateitransfers, FG254, Typ 41 zum Abbruch des Transfers (Blocknummer == 0).
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 */
public class DeBlocksMultiBlockTransfer {

    /**
     * DebugLogger für Debug-Ausgaben
     */
    private static final Debug debug = Debug.getLogger();

    /**
     * Singleton-Instanz der Klasse.
     */
    private static DeBlocksMultiBlockTransfer deBlocksMultiBlockTransfer;

    //~ FELDER ================================================================

    /**
     * Definition der DE-Bl&ouml;cke.
     */
    private int[][] _deBlocksMultiBlockTransfer;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Construktor, tut nichts. Singelton muss über Aufruf von {@link #getInstance() .#init(String) } initialisiert werden.
     */
    private DeBlocksMultiBlockTransfer() {}

    //~ GET METHODEN ==========================================================

    /**
     * Gibt die (Singleton-) Instanz der KLasse zur&uuml;ck.
     *
     * @return Instanz
     */
    public synchronized static DeBlocksMultiBlockTransfer getInstance() {
        if (deBlocksMultiBlockTransfer == null) {
            deBlocksMultiBlockTransfer = new DeBlocksMultiBlockTransfer();
        }

        return deBlocksMultiBlockTransfer;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiert die Klasse, indem die &uuml;bergebene Klasse dynamisch geladen wird.
     *
     * @param deBlocksMultiBlockTransferClassName Klassenname, der die Liste mit den zu FG/Typ/ID Kombinationen enthält, f&uuml;r
     *                                die die Zusammenstellung der Einzeltelegramme abgebrochen werden soll.
     */
    public void init(String deBlocksMultiBlockTransferClassName) {
        Class<?> classDeBlocksMultiBlockTransfer;

        try {
            classDeBlocksMultiBlockTransfer = Class.forName(deBlocksMultiBlockTransferClassName);

            /**
             * Methode im UmsetzungsModul, welche dynamisch zur Laufzeit ermittelt wird.
             */
            Method method = classDeBlocksMultiBlockTransfer.getMethod("getDeBlocksMultiBlockTransfer", new Class[] {});

            _deBlocksMultiBlockTransfer = (int[][]) method.invoke(null);
        }
        catch (ClassNotFoundException e) {
            debug.error("Klasse zur Definition von Multiblock-De-Blöcken konnte nicht geladen werden: " + deBlocksMultiBlockTransferClassName, e);
        }
        catch (NoSuchMethodException e) {
            debug.error("Methode 'getDeBlocksMultiBlockTransfer()' in der Klasse '" + deBlocksMultiBlockTransferClassName + "' existiert nicht.", e);
        }
        catch (IllegalAccessException e) {
            debug.error("Fehler beim Zugriff auf die Methode 'getDeBlocksMultiBlockTransfer()' in der Klasse '" + deBlocksMultiBlockTransferClassName + "'.", e);
        }
        catch (InvocationTargetException e) {
            debug.error("", e);
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Pr&uuml;ft, ob der DeBlock einer Fg/Typ/Id Kombination entspricht, f&uuml;r die die
     * Zusammenstellung des Einzeltelegrammes abgebrochen werden soll.
     *
     * @param deBlock DE-Block
     *
     * @return true, wenn DeBlock die Zusammenfassung abbrechen soll, sont false.
     */
    public boolean isDeBlockMultiBlockTransfer(DeBlock deBlock) {
	    for (int[] a_deBlocksMultiBlockTransfer : _deBlocksMultiBlockTransfer) {
		    if ((a_deBlocksMultiBlockTransfer[0] == deBlock.getFg()) && (a_deBlocksMultiBlockTransfer[1] == deBlock.getTyp()) && (a_deBlocksMultiBlockTransfer[2] == deBlock.getId())) {
			    return true;
		    }
	    }

        return false;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 11.09.09
