/*
 * Copyright (c) 2021 by inovat
 * innovative systeme - verkehr - tunnel - technik,
 * Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg210Default
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg210Default is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg210Default is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.osi7.conversion.Fg210Default.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg210Default.
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg210Default ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg210Default wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 *
 * Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 * Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */


package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.Eak;
import de.bsvrz.kex.tls.osi7.IllegalOsi7TelegramStructureException;
import de.bsvrz.kex.tls.osi7.TlsApplicationLayer;
import de.bsvrz.kex.tls.osi7.TlsData;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.hexdump.HexDumper;
import java.util.ArrayList;
import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Umsetungsmodul für die FG 210, Standard.
 * <p>
 * Alle Typen der FG 210 gemäß TLS 2012, erweiterte Spezifikation (gemäß Dokument `TLS2012_a6_2_xx.pdf`, welches im Repository unter
 * `Dokumente/TLS2012_a6_2_xx_D2019-12-04.pdf` abgelegt ist).
 * <p>
 * Wird standardmäßig für alle FG 210 De verwendet (Typ `typ.dePup`), bei denen keine spezielle Umsetzungsklasse in der Konfiguration angegeben wurde.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 */
public class Fg210Default extends FgAllDefault {

    /**
     * DebugLogger für Debug-Ausgaben
     */
    private static final Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    // Private ----------------------------------------------------------------

    /**
     * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "9", "29", "2", "typ.dePup"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt:
     * <ul>
     * <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger.</li>
     * <li> In der Methode {@link #update} wird für jede zur Attributgruppen/Aspekt-Kombination passende Kombination
     * eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für den
     * Beispieleintrag):<br> <code>Fg210Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
     * Dabei enthält <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li>
     * <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere
     * passende Eintrage vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere
     * ATG/ASP-Kombinationen in einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und
     * Attributgruppe eines Fachmodells):</li>
     * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </li>
     * </ul>
     */
    private final Object[][] davToTls = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
        {"atg.tlsGloDeBlockAbruf", "asp.tlsAbruf", "255", "255", "255", "typ.dePup"},                      //
        {"atg.tlsGloDeBlockSenden", "asp.tlsVorgabe", "255", "255", "255", "typ.dePup"},                   //
        {"atg.tlsGloKanalSteuerung", "asp.parameterSoll", "255", "29", "2", "typ.dePup"},                  //
        {"atg.tlsGloGeographischeKenndaten", "asp.tlsVorgabe", "255", "36", "3", "typ.dePup"},             //
        //
        {"atg.tlsPupBetriebsParameter", "asp.parameterSoll", "210", "32", "3", "typ.dePup"},               //
        {"atg.tlsPupErgänzendeBetriebsParameter", "asp.parameterSoll", "210", "33", "3", "typ.dePup"},     //
        {"atg.tlsPupKorrekturParkTaschenBelegung", "asp.tlsVorgabe", "210", "37", "3", "typ.dePup"},       //
        {"atg.tlsPupKorrekturParkPlatzBelegung", "asp.tlsVorgabe", "210", "38", "3", "typ.dePup"},         //
    };

    /**
     * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat folgenden Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "9", "29", "2", "typ.dePup"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Sendeanmeldung als Quelle.</li> <li>
     * In der Methode {@link #processDeBlock} wird für jede zum De-Block passende Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem
     * Tabelleneintrag wie folgt gebildet wird (hier für den Beispieleintrag):<br> <code>Fg210Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock,
     * Data, DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
     * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
     * De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende {@link DataDescription} für die zu sendenden Daten.</li>
     * <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden
     * entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch kann ein De-Block mehrfach ausgewertet und an verschiedene Attributgruppen
     * weitergeleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine Fachmodells):</li> <li> Ein Eintrag von "255" entspricht
     * dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] tlsToDav = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
        {"atg.tlsGloDeFehler", "asp.tlsAntwort", "210", "1", "1", "typ.dePup"},                            //
        {"atg.tlsGloDeFehler", "asp.tlsAntwort", "210", "1", "33", "typ.dePup"},                           //
        {"atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.dePup"},                   //
        {"atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "34", "typ.dePup"},                  //
        {"atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.dePup"},             //
        {"atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "35", "typ.dePup"},            //
        //
        {"atg.tlsPupErgänzendeDeFehlerMeldung", "asp.tlsAntwort", "210", "14", "1", "typ.dePup"},          //
        {"atg.tlsPupErgänzendeDeFehlerMeldung", "asp.tlsAntwort", "210", "14", "33", "typ.dePup"},         //
        {"atg.tlsPupNegativeQuittung", "asp.tlsAntwort", "210", "16", "2", "typ.dePup"},                   //
        {"atg.tlsPupNegativeQuittung", "asp.tlsAntwort", "210", "16", "34", "typ.dePup"},                  //
        {"atg.tlsPupBetriebsParameter", "asp.parameterIst", "210", "32", "3", "typ.dePup"},                //
        {"atg.tlsPupBetriebsParameter", "asp.parameterIst", "210", "32", "35", "typ.dePup"},               //
        {"atg.tlsPupErgänzendeBetriebsParameter", "asp.parameterIst", "210", "33", "3", "typ.dePup"},      //
        {"atg.tlsPupErgänzendeBetriebsParameter", "asp.parameterIst", "210", "33", "35", "typ.dePup"},     //
        {"atg.tlsPupParkPlatzBelegungVersion0", "asp.tlsAntwort", "210", "48", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion0", "asp.tlsAntwort", "210", "48", "36", "typ.dePup"},         //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "49", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "49", "36", "typ.dePup"},         //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "50", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "50", "36", "typ.dePup"},         //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "51", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "51", "36", "typ.dePup"},         //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "52", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "52", "36", "typ.dePup"},         //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "53", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion1Bis5", "asp.tlsAntwort", "210", "53", "36", "typ.dePup"},         //
        {"atg.tlsPupParkPlatzBelegungVersion6", "asp.tlsAntwort", "210", "54", "4", "typ.dePup"},          //
        {"atg.tlsPupParkPlatzBelegungVersion6", "asp.tlsAntwort", "210", "54", "36", "typ.dePup"},         //
        {"atg.tlsPupFreieParkPlatzBereiche", "asp.tlsAntwort", "210", "60", "4", "typ.dePup"},             //
        {"atg.tlsPupFreieParkPlatzBereiche", "asp.tlsAntwort", "210", "60", "36", "typ.dePup"},            //
        {"atg.tlsPupFreieParkPlatzBereichsZonen", "asp.tlsAntwort", "210", "61", "4", "typ.dePup"},        //
        {"atg.tlsPupFreieParkPlatzBereichsZonen", "asp.tlsAntwort", "210", "61", "36", "typ.dePup"},       //
        {"atg.tlsPupBelegungParkStandsReihen", "asp.tlsAntwort", "210", "62", "4", "typ.dePup"},           //
        {"atg.tlsPupBelegungParkStandsReihen", "asp.tlsAntwort", "210", "62", "36", "typ.dePup"},          //
        {"atg.tlsPupFahrzeugIdentifikation", "asp.tlsAntwort", "210", "63", "4", "typ.dePup"},             //
        {"atg.tlsPupFahrzeugIdentifikation", "asp.tlsAntwort", "210", "63", "36", "typ.dePup"},            //
    };

    /**
     * Erzeugt ein Objekt vom Typ Fg210Default
     *
     * @param daf Datenverteiler
     * @param eak Das übergeordnete EAK dieses DE
     * @param de  Das DE
     */
    public Fg210Default(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
        subscribeAllData(tlsToDav, davToTls);
    }

    ClientDavInterface _daf;
    /** ACHTUNG: Nur zum Testen!. */
    /**
     *
     * @param daf Datenverteiler
     */
    public Fg210Default(ClientDavInterface daf) {
        super();
        _daf = daf;
    }


    // ====================================================================================
    // Typ 1 DE-Fehler
    // ====================================================================================

    /**
     * FG 210: Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Beschreibung siehe [TLS 2012].
     * <p>
     * Ergänzt globale Behandlung De-Fehler in {@link #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}.
     * <p>
     * Beim Empfang einer De-Gutmeldung wird die DE initialisiert, indem folgende Telegramme verschickt werden:
     * <ul>
     * <li>Typ 29: PupKanalsteuerung</li>
     * <li>Typ 32: PupBetriebsParameter</li>
     * <li>Typ 33: PupErgänzendeBetriebsParameter</li>
     * </ul>
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);
        IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();

        if (DEFehlerStatus != null) {
            debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());

            if (DEFehlerStatus.getName().equals("ok")) {
                sendInitialDeBlock(deBlock.getKey(), 29, 2);  // PupKanalsteuerung senden
                sendInitialDeBlock(deBlock.getKey(), 32, 3);  // PupBetriebsParameterZuflussRegelung senden
                sendInitialDeBlock(deBlock.getKey(), 33, 3);  // PupBetriebsParameterAlinea1 senden
            } else {
                List<De> deList = new ArrayList<>();

                deList.add(_de);
                TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
            }
        }

        return resultData;
    }

    /**
     * FG 210: Typ: 1 Id: 33 Name: DE-Fehler (Antwort). Standardverarbeitung wie bei Id 1,
     * {@link #Fg210Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock,
     * Data, DataDescription) ()}, aber als "nachgelieferte" Daten an den DaV übergeben. Zudem werden bei nachgelieferten Daten keine Sequenzen
     * verschickt.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ001Id033AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 1, aber:
        // 1. als "nachgelieferte" Daten an den DaV übergeben.
        // 2. bei nachgelieferten Daten werden keine Sequenzen verschickt.
        ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // ====================================================================================
    // Typ 14 Ergänzende Fehlermeldung
    // ====================================================================================

    /**
     * FG 210: Typ: 14 Id: 1  Name: Ergänzende DE-Fehlermeldung (Antwort). Beschreibung siehe [TLS 2012].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ014Id001AtgTlsPupErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
        int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Hersteller").set(_Herstellercode_);

        // Fehler-Bytes aus Telegramm -----------------------------------------------------------------------------------
        deBlock.getDeData().readUnsignedByteAsInt();  // Anzahl der folgenden Bytes überlesen, da immer == 4 !

        int _TlsFehler1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _TlsFehler2_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _TlsFehler3_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _TlsFehler4_ = deBlock.getDeData().readUnsignedByteAsInt();
        Data.NumberArray _TlsFehler_ = data.getItem("FehlercodesTLS").getUnscaledArray("Fehler");

        _TlsFehler_.setLength(4);
        _TlsFehler_.getValue(0).set(_TlsFehler1_);
        _TlsFehler_.getValue(1).set(_TlsFehler2_);
        _TlsFehler_.getValue(2).set(_TlsFehler3_);
        _TlsFehler_.getValue(3).set(_TlsFehler4_);

        // Hersteller Fehler-Bytes aus Telegramm -----------------------------------------------------------------------
        int count = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _HerstellerFehler_ = new int[count];

        for (int i = 0; i < count; i++) {
            _HerstellerFehler_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        Data.NumberArray _Fehler_ = data.getItem("FehlercodesHersteller").getUnscaledArray("Fehler");

        _Fehler_.setLength(count);

        for (int i = 0; i < count; i++) {
            _Fehler_.getValue(i).set(_HerstellerFehler_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        // ACHTUNG: Hier nur zum Testen
        resultData = new ResultData(_daf.getLocalConfigurationAuthority(), dataDescription, 1234, data);
        //resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 14 Id: 33 Name: Ergänzende DE-Fehlermeldung (Antwort). Standardverarbeitung wie bei Id 2, {@link
     * #Fg210Typ014Id001AtgTlsPupErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ014Id033AtgTlsPupErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ014Id001AtgTlsPupErgänzendeDeFehlerMeldungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // ====================================================================================
    // Typ 16 Negative Quittung
    // ====================================================================================

    /**
     * FG 210: Typ: 16 Id: 2 Name: Negative Quittung (Antwort). Beschreibung siehe [TLS 2012].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ016Id002AtgTlsPupNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return super.Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);
    }

    /**
     * FG 210: Typ: 16 Id: 34 Name: Negative Quittung (Antwort). Standardverarbeitung wie bei ID 2, {@link
     * #Fg210Typ016Id002AtgTlsPupNegativeQuittungAspTlsAntwort(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ016Id034AtgTlsPupNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 2, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ016Id002AtgTlsPupNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // ====================================================================================
    // Typ 32 Betriebsparameter Parken und Parkplatzüberwachung
    // ====================================================================================

    /**
     * FG 210: Typ: 32 Id: 3 Name: Betriebsparameter Parken und Parkplatzüberwachung (Antwort). Beschreibung siehe [TLS 2012].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ032Id003AtgTlsPupBetriebsParameterAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _erfassungsPeriodenDauer_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _verfahrenVersionTyp_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Übertragungsverfahren, Version und Typ aus dem Wert _verfahrenVersionTyp_ ermitteln
        int _verfahren_ = _verfahrenVersionTyp_ & 15;
        int _version_ = _verfahrenVersionTyp_ >> 4 & 7;
        int _typ_ = _verfahrenVersionTyp_ >> 7 & 1;

        data.getUnscaledValue("ErfassungsPeriodenDauer").set(_erfassungsPeriodenDauer_);
        data.getUnscaledValue("ÜbertragungsVerfahren").set(_verfahren_);
        data.getUnscaledValue("VersionErgebnisMeldung").set(_version_);
        data.getUnscaledValue("Typ63").set(_typ_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 32 Id: 3 Name: Betriebsparameter Parken und Parkplatzüberwachung (Senden). Beschreibung siehe [TLS 2012].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg210Typ032Id003AtgTlsPupBetriebsParameterAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data data = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _erfassungsPeriodenDauer_ = data.getUnscaledValue("ErfassungsPeriodenDauer").intValue();
        int _verfahren_ = data.getUnscaledValue("ÜbertragungsVerfahren").intValue();
        int _version_ = data.getUnscaledValue("VersionErgebnisMeldung").intValue();
        int _typ_ = data.getUnscaledValue("Typ63").intValue();

        // Übertragungsverfahren, Version und Typ zum Byte _verfahrenVersionTyp_ zusammensetzen
        int _verfahrenVersionTyp_ = _verfahren_ | _version_ << 4 | _typ_ << 7;

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(3);

        deApplicationData.writeIntAsLowHighByte(_erfassungsPeriodenDauer_);
        deApplicationData.writeIntAsByte(_verfahrenVersionTyp_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 32, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        } catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 210: Typ: 32 Id: 35 Name: Betriebsparameter Parken und Parkplatzüberwachung (Antwort). Standardverarbeitung wie bei Id 3, {@link
     * #Fg210Typ032Id003AtgTlsPupBetriebsParameterAspParameterIst(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ032Id035AtgTlsPupBetriebsParameterZuflussRegelungAspParameterIst(DeBlock deBlock, Data data,
                                                                                               DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ032Id003AtgTlsPupBetriebsParameterAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // ====================================================================================
    // Typ 33
    // ====================================================================================

    /**
     * FG 210: Typ: 33 Id: 3 Name: Ergänzende Betriebsparameter Parken und Parkplatzüberwachung (Antwort). Beschreibung siehe [TLS 2012].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ033Id003AtgTlsPupErgänzendeBetriebsParameterAspParameterIst(DeBlock deBlock, Data data,
                                                                                          DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _kfzLaengenGrenzWert_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _kfzHoehenGrenzWert_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _kfzBreitenGrenzWert_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _maxAnzahlStVOkonformerParkPlaetze_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _maxAnzahlNichtStVOkonformerParkPlaetze_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlParkPlatzBereiche_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _reserve1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _reserve2_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _mindestRestLaengeParkStandsReihen_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _parameterParkStandsReihen_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Parameter in _offsetMindestRestLaengeParkStandsReihen_ und _meldungsIntervallParkStandsReihen_
        int _offsetMindestRestLaengeParkStandsReihen_ = _parameterParkStandsReihen_ & 15;
        int _meldungsIntervallParkStandsReihen_ = _parameterParkStandsReihen_ >> 4 & 15;

        // Fixen Teil des Telegramms in Ziel-ATG übertragen...
        data.getUnscaledValue("KfzLängenGrenzWert").set(_kfzLaengenGrenzWert_);
        data.getUnscaledValue("KfzHöhenGrenzWert").set(_kfzHoehenGrenzWert_);
        data.getUnscaledValue("KfzBreitenGrenzWert").set(_kfzBreitenGrenzWert_);
        data.getUnscaledValue("MaxAnzahlStVOkonformerParkPlätze").set(_maxAnzahlStVOkonformerParkPlaetze_);
        data.getUnscaledValue("MaxAnzahlNichtStVOkonformerParkPlätze").set(_maxAnzahlNichtStVOkonformerParkPlaetze_);
        data.getUnscaledValue("AnzahlParkPlatzBereiche").set(_anzahlParkPlatzBereiche_);
        data.getUnscaledValue("Reserve1").set(_reserve1_);
        data.getUnscaledValue("Reserve2").set(_reserve2_);
        data.getUnscaledValue("MindestRestLängeParkStandsReihen").set(_mindestRestLaengeParkStandsReihen_);
        data.getUnscaledValue("OffsetMindestRestLängeParkStandsReihen").set(_offsetMindestRestLaengeParkStandsReihen_);
        data.getUnscaledValue("MeldungsIntervallParkStandsReihen").set(_meldungsIntervallParkStandsReihen_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 33 Id: 3 Name: Ergänzende Betriebsparameter Parken und Parkplatzüberwachung (Senden). Beschreibung siehe [TLS 2012].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg210Typ033Id003AtgTlsPupErgänzendeBetriebsParameterAspParameterSoll(ResultData result) {
        DeBlock deBlock = null;
        Data data = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung
        int _kfzLaengenGrenzWert_ = data.getUnscaledValue("KfzLängenGrenzWert").intValue();
        int _kfzHoehenGrenzWert_ = data.getUnscaledValue("KfzHöhenGrenzWert").intValue();
        int _kfzBreitenGrenzWert_ = data.getUnscaledValue("KfzBreitenGrenzWert").intValue();
        int _maxAnzahlStVOkonformerParkPlaetze_ = data.getUnscaledValue("MaxAnzahlStVOkonformerParkPlätze").intValue();
        int _maxAnzahlNichtStVOkonformerParkPlaetze_ = data.getUnscaledValue("MaxAnzahlNichtStVOkonformerParkPlätze").intValue();
        int _anzahlParkPlatzBereiche_ = data.getUnscaledValue("AnzahlParkPlatzBereiche").intValue();
        int _reserve1_ = data.getUnscaledValue("Reserve1").intValue();
        int _reserve2_ = data.getUnscaledValue("Reserve2").intValue();
        int _mindestRestLaengeParkStandsReihen_ = data.getUnscaledValue("MindestRestLängeParkStandsReihen").intValue();
        int _offsetMindestRestLaengeParkStandsReihen_ = data.getUnscaledValue("OffsetMindestRestLängeParkStandsReihen").intValue();
        int _meldungsIntervallParkStandsReihen_ = data.getUnscaledValue("MeldungsIntervallParkStandsReihen").intValue();

        // Parameter aus _offsetMindestRestLaengeParkStandsReihen_ und _meldungsIntervallParkStandsReihen_ zusammenstellen
        int _parameterParkStandsReihen_ = _offsetMindestRestLaengeParkStandsReihen_ | _meldungsIntervallParkStandsReihen_ << 4;

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(12);

        // Fixen Teil des Telegramms in DE-Block übertragen...
        deApplicationData.writeIntAsByte(_kfzLaengenGrenzWert_);
        deApplicationData.writeIntAsByte(_kfzHoehenGrenzWert_);
        deApplicationData.writeIntAsByte(_kfzBreitenGrenzWert_);
        deApplicationData.writeIntAsByte(_maxAnzahlStVOkonformerParkPlaetze_);
        deApplicationData.writeIntAsByte(_maxAnzahlNichtStVOkonformerParkPlaetze_);
        deApplicationData.writeIntAsByte(_anzahlParkPlatzBereiche_);
        deApplicationData.writeIntAsByte(_reserve1_);
        deApplicationData.writeIntAsByte(_reserve2_);
        deApplicationData.writeIntAsByte(_mindestRestLaengeParkStandsReihen_);
        deApplicationData.writeIntAsByte(_parameterParkStandsReihen_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 33, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        } catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    /**
     * FG 210: Typ: 33 Id: 3 Name: Ergänzende Betriebsparameter Parken und Parkplatzüberwachung (Antwort). Standardverarbeitung wie bei Id 3, {@link
     * #Fg210Typ033Id003AtgTlsPupErgänzendeBetriebsParameterAspParameterIst(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den
     * DaV übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ033Id035AtgTlsPupErgänzendeBetriebsParameterAspParameterIst(DeBlock deBlock, Data data,
                                                                                          DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 3, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ033Id003AtgTlsPupErgänzendeBetriebsParameterAspParameterIst(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }

    // ====================================================================================
    // Typ 37 Korrektur der Parktaschenbelegung
    // ====================================================================================

    /**
     * FG 210: Typ: 37 Id: 3 Name: Korrektur der Parktaschenbelegung (Senden). Beschreibung siehe [TLS 2012].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg210Typ037Id003AtgTlsPupKorrekturParkTaschenBelegungAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;
        Data data = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung

        int _anzahlParktaschen_;
        int _anzahlSensoren_;
        int _korrekturBelegung_;
        // Anzahl der nachfolgenden Parktaschen auslesen...

        // Datenfeld auf passende Länge initialisieren
        _anzahlParktaschen_ = data.getArray("ParkTaschen").getLength();

        // DeBlock als TlsData aufbauen. Wird hier "nur" auf Länge = 1 gesetzt, da Anzahl der Sensoren je Parktasche vorab nicht ermittelt werden
        // kann.
        TlsData deApplicationData = new TlsData(1);

        // Byte 4
        deApplicationData.writeIntAsByte(_anzahlParktaschen_);

        // Schleife über die Parktaschen direkt auslesen und in DE-Block schreiben
        for (int i = 0; i < _anzahlParktaschen_; i++) {
            _anzahlSensoren_ = data.getArray("ParkTaschen").getItem(i).getArray("Sensor").getLength();
            deApplicationData.writeIntAsByte(_anzahlSensoren_);

            for (int j = 0; j < _anzahlSensoren_; j++) {
                _korrekturBelegung_ =
                    data.getArray("ParkTaschen").getItem(i).getArray("Sensor").getItem(j).getUnscaledValue("KorrekturBelegung").intValue();
                deApplicationData.writeIntAsByte(_korrekturBelegung_);
            }
        }

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 37, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        } catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }

    // ====================================================================================
    // Typ 38 Korrektur der Parkplatzbelegung
    // ====================================================================================

    /**
     * FG 210: Typ: 38 Id: 3 Name: Korrektur der Parkplatzbelegung (Senden). Beschreibung siehe [TLS 2012].
     *
     * @param result Der vom Datenverteiler empfangene Datensatz, der in einen De-Block konvertiert wird.
     *
     * @return De-Block mit den konvertierten Daten.
     */
    public DeBlock Fg210Typ038Id003AtgTlsPupKorrekturParkPlatzBelegungAspTlsVorgabe(ResultData result) {
        DeBlock deBlock = null;
        Data data = result.getData();  // Datensatz holen

        // Datenblock beseht aus folgenden Inhalten / Umsetzung

        int _fahrzeugKlassenCode_ = data.getUnscaledValue("FahrzeugKlassenCode").intValue();
        int _korrekturMethode_ = data.getUnscaledValue("KorrekturMethode").intValue();
        int _korrekturFaktor_ = data.getUnscaledValue("KorrekturFaktor").intValue();

        // DeBlock als TlsData aufbauen...
        TlsData deApplicationData = new TlsData(4);

        deApplicationData.writeIntAsByte(_fahrzeugKlassenCode_);
        deApplicationData.writeIntAsByte(_korrekturMethode_);
        deApplicationData.writeIntAsLowHighByte(_korrekturFaktor_);

        // DeBlock erzeugen
        try {
            deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 38, 3, _de.getDeChannel());
            debug.finest("TLS-DE-Block: " + HexDumper.toString(deBlock.getTlsDeDataBlock()));
        } catch (IllegalOsi7TelegramStructureException e) {
            debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
        }

        return deBlock;
    }
    // ====================================================================================
    // Typ 48 Parkplatzbelegung Version 0
    // ====================================================================================

    /**
     * FG 210: Typ: 48 Id: 4 Name: Parkplatzbelegung Version 0 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ048Id004AtgTlsPupParkPlatzBelegungVersion0AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _anzahlParkTaschen_ = deBlock.getDeData().readUnsignedByteAsInt();

        int _anzahlSensoren_;
        int _sensor_;
        int _belegung_;
        int _zustand_;
        int _reserve_;
        int _herstellerInfo_;

        data.getArray("ParkTasche").setLength(_anzahlParkTaschen_);
        for (int i = 0; i < _anzahlParkTaschen_; i++) {
            Data parkTasche = data.getArray("ParkTasche").getItem(i);

            _anzahlSensoren_ = deBlock.getDeData().readUnsignedByteAsInt();

            for (int j = 0; j < _anzahlSensoren_; j++) {

                _sensor_ = deBlock.getDeData().readUnsignedByteAsInt();
                _belegung_ = _sensor_ & 1;
                _zustand_ = _sensor_ >> 1 & 1;
                _reserve_ = _sensor_ >> 2 & 3;
                _herstellerInfo_ = _sensor_ >> 4 & 15;

                parkTasche.getArray("Sensor").getItem(j).getUnscaledValue("Belegung").set(_belegung_);
                parkTasche.getArray("Sensor").getItem(j).getUnscaledValue("Zustand").set(_zustand_);
                parkTasche.getArray("Sensor").getItem(j).getUnscaledValue("Reserve").set(_reserve_);
                parkTasche.getArray("Sensor").getItem(j).getUnscaledValue("HerstellerInfo").set(_herstellerInfo_);

            }
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 48 Id: 36 Name: Parkplatzbelegung Version 0 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ048Id004AtgTlsPupParkPlatzBelegungVersion0AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ048Id036AtgTlsPupParkPlatzBelegungVersion0AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ048Id004AtgTlsPupParkPlatzBelegungVersion0AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 49 - 53 Parkplatzbelegung Version 1 - 5
    // ====================================================================================

    /**
     * FG 210: Verarbeitet alls Telegrammversionen der TLS Parkplatzbelegung Version 1-5 (Typ 49-53). Genaue Beschreibung bzg. Aufbau der Telegramme
     * in den einzelnen Versionen siehe TLS. [TLS 2012].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData TlsPupParkPlatzBelegungVersion1Bis5(DeBlock deBlock, Data data, DataDescription dataDescription) {

        ResultData resultData;

        final String NICHT_ERMITTELBAR = "nicht ermittelbar";
        int _anzahlFz_;

        // Daten vorbelegen mit nicht ermittelbar
        data.getUnscaledValue("Kfz").setText(NICHT_ERMITTELBAR);           // Verwendung bei Typ 49
        data.getUnscaledValue("PkwÄ").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 50
        data.getUnscaledValue("LkwÄ").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 50
        data.getUnscaledValue("LVoÄ").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 51
        data.getUnscaledValue("SGVÄ").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 51
        data.getUnscaledValue("BPAÄ").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 51
        data.getUnscaledValue("nkKfz").setText(NICHT_ERMITTELBAR);         // Verwendung bei Typ 52, 53
        data.getUnscaledValue("PkwG").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 52
        data.getUnscaledValue("Lkw").setText(NICHT_ERMITTELBAR);           // Verwendung bei Typ 52
        data.getUnscaledValue("LkwK").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 52
        data.getUnscaledValue("Bus").setText(NICHT_ERMITTELBAR);           // Verwendung bei Typ 52, 53
        data.getUnscaledValue("PkwA").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 52, 53
        data.getUnscaledValue("MRad").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 53
        data.getUnscaledValue("Pkw").setText(NICHT_ERMITTELBAR);           // Verwendung bei Typ 53
        data.getUnscaledValue("Lfw").setText(NICHT_ERMITTELBAR);           // Verwendung bei Typ 53
        data.getUnscaledValue("LkwA").setText(NICHT_ERMITTELBAR);          // Verwendung bei Typ 53
        data.getUnscaledValue("SattelKfz").setText(NICHT_ERMITTELBAR);     // Verwendung bei Typ 53

        switch (deBlock.getTyp()) {
            case 49:
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("Kfz").set(_anzahlFz_);           // Verwendung bei Typ 49
                break;
            case 50:
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("PkwÄ").set(_anzahlFz_);          // Verwendung bei Typ 50
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("LkwÄ").set(_anzahlFz_);          // Verwendung bei Typ 50
                break;
            case 51:
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("LVoÄ").set(_anzahlFz_);          // Verwendung bei Typ 51
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("SGVÄ").set(_anzahlFz_);          // Verwendung bei Typ 51
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("BPAÄ").set(_anzahlFz_);          // Verwendung bei Typ 51
                break;
            case 52:
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("nkKfz").set(_anzahlFz_);         // Verwendung bei Typ 52, 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("PkwG").set(_anzahlFz_);          // Verwendung bei Typ 52
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("Lkw").set(_anzahlFz_);           // Verwendung bei Typ 52
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("LkwK").set(_anzahlFz_);          // Verwendung bei Typ 52
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("Bus").set(_anzahlFz_);           // Verwendung bei Typ 52, 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("PkwA").set(_anzahlFz_);          // Verwendung bei Typ 52, 53
                break;
            case 53:
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("nkKfz").set(_anzahlFz_);         // Verwendung bei Typ 52, 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("MRad").set(_anzahlFz_);          // Verwendung bei Typ 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("Pkw").set(_anzahlFz_);           // Verwendung bei Typ 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("Lfw").set(_anzahlFz_);           // Verwendung bei Typ 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("LkwA").set(_anzahlFz_);          // Verwendung bei Typ 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("SattelKfz").set(_anzahlFz_);     // Verwendung bei Typ 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("Bus").set(_anzahlFz_);           // Verwendung bei Typ 52, 53
                _anzahlFz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
                data.getUnscaledValue("PkwA").set(_anzahlFz_);          // Verwendung bei Typ 52, 53
                break;

            default:
                debug.warning("Unbekannter Pup-Parkplatzbelegungs-Typ (49-53): " + deBlock.getTyp());
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    // ====================================================================================
    // Typ 49 Parkplatzbelegung Version 1
    // ====================================================================================

    /**
     * FG 210: Typ: 49 Id: 4 Name: Parkplatzbelegung Version 1 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ049Id004AtgTlsPupParkPlatzBelegungVersion1AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return TlsPupParkPlatzBelegungVersion1Bis5(deBlock, data, dataDescription);
    }

    /**
     * FG 210: Typ: 49 Id: 36 Name: Parkplatzbelegung Version 1 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ049Id004AtgTlsPupParkPlatzBelegungVersion1AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ049Id036AtgTlsPupParkPlatzBelegungVersion1AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ049Id004AtgTlsPupParkPlatzBelegungVersion1AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;

    }

    // ====================================================================================
    // Typ 50 Parkplatzbelegung Version 2
    // ====================================================================================

    /**
     * FG 210: Typ: 50 Id: 4 Name: Parkplatzbelegung Version 2 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ050Id004AtgTlsPupParkPlatzBelegungVersion2AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return TlsPupParkPlatzBelegungVersion1Bis5(deBlock, data, dataDescription);
    }

    /**
     * FG 210: Typ: 50 Id: 36 Name: Parkplatzbelegung Version 2 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ050Id004AtgTlsPupParkPlatzBelegungVersion2AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ050Id036AtgTlsPupParkPlatzBelegungVersion2AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ050Id004AtgTlsPupParkPlatzBelegungVersion2AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 51 Parkplatzbelegung Version 3
    // ====================================================================================

    /**
     * FG 210: Typ: 51 Id: 4 Name: Parkplatzbelegung Version 3 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ051Id004AtgTlsPupParkPlatzBelegungVersion3AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return TlsPupParkPlatzBelegungVersion1Bis5(deBlock, data, dataDescription);
    }

    /**
     * FG 210: Typ: 51 Id: 36 Name: Parkplatzbelegung Version 3 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ051Id004AtgTlsPupParkPlatzBelegungVersion3AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ051Id036AtgTlsPupParkPlatzBelegungVersion3AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ051Id004AtgTlsPupParkPlatzBelegungVersion3AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;

    }

    // ====================================================================================
    // Typ 52 Parkplatzbelegung Version 4
    // ====================================================================================

    /**
     * FG 210: Typ: 52 Id: 4 Name: Parkplatzbelegung Version 4 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ052Id004AtgTlsPupParkPlatzBelegungVersion4AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return TlsPupParkPlatzBelegungVersion1Bis5(deBlock, data, dataDescription);
    }

    /**
     * FG 210: Typ: 52 Id: 36 Name: Parkplatzbelegung Version 4 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ052Id004AtgTlsPupParkPlatzBelegungVersion4AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ052Id036AtgTlsPupParkPlatzBelegungVersion4AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ052Id004AtgTlsPupParkPlatzBelegungVersion4AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 53 Parkplatzbelegung Version 5
    // ====================================================================================

    /**
     * FG 210: Typ: 53 Id: 4 Name: Parkplatzbelegung Version 5 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ053Id004AtgTlsPupParkPlatzBelegungVersion5AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        return TlsPupParkPlatzBelegungVersion1Bis5(deBlock, data, dataDescription);
    }

    /**
     * FG 210: Typ: 53 Id: 36 Name: Parkplatzbelegung Version 5 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ053Id004AtgTlsPupParkPlatzBelegungVersion5AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ053Id036AtgTlsPupParkPlatzBelegungVersion5AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ053Id004AtgTlsPupParkPlatzBelegungVersion5AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 54 Parkplatzbelegung Version 6
    // ====================================================================================

    /**
     * FG 210: Typ: 54 Id: 4 Name: Parkplatzbelegung Version 6 (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ054Id004AtgTlsPupParkPlatzBelegungVersion6AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _anzahlNkKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlMRad_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlPkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlLfw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlLkw_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlLkwA_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlSattelKfz_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlBusse_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _anzahlPkwA_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

        data.getUnscaledValue("AnzahlNkKfz").set(_anzahlNkKfz_);
        data.getUnscaledValue("AnzahlMRad").set(_anzahlMRad_);
        data.getUnscaledValue("AnzahlPkw").set(_anzahlPkw_);
        data.getUnscaledValue("AnzahlLkw").set(_anzahlLkw_);
        data.getUnscaledValue("AnzahlLfw").set(_anzahlLfw_);
        data.getUnscaledValue("AnzahlLkw").set(_anzahlLkw_);
        data.getUnscaledValue("AnzahlLkwA").set(_anzahlLkwA_);
        data.getUnscaledValue("AnzahlSattelKfz").set(_anzahlSattelKfz_);
        data.getUnscaledValue("AnzahlBusse").set(_anzahlBusse_);
        data.getUnscaledValue("AnzahlPkwA").set(_anzahlPkwA_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 54 Id: 36 Name: Parkplatzbelegung Version 6 (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ054Id004AtgTlsPupParkPlatzBelegungVersion6AspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ054Id036AtgTlsPupParkPlatzBelegungVersion6AspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ054Id004AtgTlsPupParkPlatzBelegungVersion6AspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 60 Freie Parkplatzbereiche
    // ====================================================================================

    /**
     * FG 210: Typ: 60 Id: 4 Name: Freie Parkplatzbereiche (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ060Id004AtgTlsPupFreieParkPlatzBereicheAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _parkPlatzBereich_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _typParkPlatzBereich_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _maxParkPlaetze_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _qfPB_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _qbfPB_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _reserve1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _reserve2_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("ParkPlatzBereich").set(_parkPlatzBereich_);
        data.getUnscaledValue("TypParkPlatzBereich").set(_typParkPlatzBereich_);
        data.getUnscaledValue("MaxParkPlätze").set(_maxParkPlaetze_);
        data.getUnscaledValue("QfPB").set(_qfPB_);
        data.getUnscaledValue("QbfPB").set(_qbfPB_);
        data.getUnscaledValue("Reserve1").set(_reserve1_);
        data.getUnscaledValue("Reserve2").set(_reserve2_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 60 Id: 36 Name: Freie Parkplatzbereiche (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ060Id004AtgTlsPupFreieParkPlatzBereicheAspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ060Id036AtgTlsPupFreieParkPlatzBereicheAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ060Id004AtgTlsPupFreieParkPlatzBereicheAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 61 Freie Parkplatzbereichszonen
    // ====================================================================================

    /**
     * FG 210: Typ: 61 Id: 4 Name: Freie Parkplatzbereichszonen (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ061Id004AtgTlsPupFreieParkPlatzBereichsZonenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _parkPlatzBereich_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _parkPlatzBereichsZone_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _maxParkPlaetze_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _qfPB_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _qbfPB_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _reserve1_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _reserve2_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _anzSegmente_ = deBlock.getDeData().readUnsignedByteAsInt();
        int[] _segmentVerweilDauer_ = new int[_anzSegmente_];

        for (int i = 0; i < _anzSegmente_; i++) {
            _segmentVerweilDauer_[i] = deBlock.getDeData().readUnsignedByteAsInt();
        }

        data.getUnscaledValue("ParkPlatzBereich").set(_parkPlatzBereich_);
        data.getUnscaledValue("ParkPlatzBereichsZone").set(_parkPlatzBereichsZone_);
        data.getUnscaledValue("MaxParkPlätze").set(_maxParkPlaetze_);
        data.getUnscaledValue("QfPB").set(_qfPB_);
        data.getUnscaledValue("QbfPB").set(_qbfPB_);
        data.getUnscaledValue("Reserve1").set(_reserve1_);
        data.getUnscaledValue("Reserve2").set(_reserve2_);

        data.getUnscaledArray("SegmentVerweilDauer").setLength(_anzSegmente_);
        for (int i = 0; i < _anzSegmente_; i++) {
            data.getUnscaledArray("SegmentVerweilDauer").getValue(i).set(_segmentVerweilDauer_[i]);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 61 Id: 36 Name: Freie Parkplatzbereichszonen (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ061Id004AtgTlsPupFreieParkPlatzBereichsZonenAspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ061Id036AtgTlsPupFreieParkPlatzBereichsZonenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ061Id004AtgTlsPupFreieParkPlatzBereichsZonenAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 62 Belegung der Parkstandsreihen
    // ====================================================================================

    /**
     * FG 210: Typ: 62 Id: 4 Name: Belegung ParkStandsReihen (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ062Id004AtgTlsPupBelegungParkStandsReihenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _anzahlParkStandsReihen_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _restLaenge_;
        int _belegung_;
        int _status_;
        int _reserve_;
        int _gesamt_;

        data.getArray("ZustandParkStandsReihe").setLength(_anzahlParkStandsReihen_);

        for (int i = 0; i < _anzahlParkStandsReihen_; i++) {
            _gesamt_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
            _restLaenge_ = _gesamt_ & 2047;
            _belegung_ = _gesamt_ >> 11 & 3;
            _status_ = _gesamt_ >> 13 & 3;
            _reserve_ = _gesamt_ >> 15 & 1;
            data.getArray("ZustandParkStandsReihe").getItem(i).getUnscaledValue("RestLänge").set(_restLaenge_);
            data.getArray("ZustandParkStandsReihe").getItem(i).getUnscaledValue("BelegungsZustand").set(_belegung_);
            data.getArray("ZustandParkStandsReihe").getItem(i).getUnscaledValue("SensorDetektorStatus").set(_status_);
            data.getArray("ZustandParkStandsReihe").getItem(i).getUnscaledValue("Reserve").set(_reserve_);
        }

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 62 Id: 36 Name: Belegung ParkStandsReihen (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ062Id004AtgTlsPupBelegungParkStandsReihenAspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ062Id036AtgTlsPupBelegungParkStandsReihenAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ062Id004AtgTlsPupBelegungParkStandsReihenAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

    // ====================================================================================
    // Typ 63 Fahrzeugidentifikation
    // ====================================================================================

    /**
     * FG 210: Typ: 63 Id: 4 Name: Fahrzeug Identifikation (Antwort). Beschreibung siehe [TLS 2011].
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ063Id004AtgTlsPupFahrzeugIdentifikationAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData;

        // Daten aus Telegramm -----------------------------------------------------------------------
        int _id_ = deBlock.getDeData().readUnsignedLowHighExtraHighVeryHighByteAsInt();
        int _detektionsByte_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _fahrzeugKlassenCode_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _geschwindigkeit_ = deBlock.getDeData().readUnsignedByteAsInt();
        int _belegtZeit_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
        int _fahrzeugLaenge_ = deBlock.getDeData().readUnsignedByteAsInt();

        // Bestandteile des _detektionsByte_ extrahieren
        int _detektionsZaehler_ = _detektionsByte_ & 63;
        int _detektionsOrt_ = _detektionsByte_ >> 6 & 3;

        data.getUnscaledValue("ID").set(_id_);
        data.getUnscaledValue("DetektionsZähler").set(_detektionsZaehler_);
        data.getUnscaledValue("DetektionsOrt").set(_detektionsOrt_);
        data.getUnscaledValue("FahrzeugKlassenCode").set(_fahrzeugKlassenCode_);
        data.getUnscaledValue("Geschwindigkeit").set(_geschwindigkeit_);
        data.getUnscaledValue("BelegtZeit").set(_belegtZeit_);
        data.getUnscaledValue("FahrzeugLänge").set(_fahrzeugLaenge_);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * FG 210: Typ: 63 Id: 36 Name: Fahrzeug Identifikation (Antwort). Beschreibung siehe [TLS 2011]. Standardverarbeitung wie bei Id 4, {@link
     * #Fg210Typ063Id004AtgTlsPupFahrzeugIdentifikationAspTlsZustand(DeBlock, Data, DataDescription)}, aber als "nachgelieferte" Daten an den DaV
     * übergeben.
     *
     * @param deBlock         DE-Block, der konvertiert werden soll.             +
     * @param data            Der passende Datensatz für die Ziel-Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg210Typ063Id036AtgTlsPupFahrzeugIdentifikationAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV übergeben.
        ResultData resultData = Fg210Typ063Id004AtgTlsPupFahrzeugIdentifikationAspTlsZustand(deBlock, data, dataDescription);

        if (resultData != null) {
            return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
        }
        return null;
    }

}


